import Paypal from "paypal-rest-sdk";
import Stripe from "stripe";
import { EpicPaypalPaymentsOneTime } from "./paypal/payments/one-time";
import { EpicStripePaymentsOneTime } from "./stripe/payments/one-time";
export declare type Mode = "Live" | "Test";
export declare type Methods = "Paypal" | "Stripe";
export declare type Types = "One-Time";
export declare type Credentials = {
    [key in Mode]?: {
        SK: string;
        PK?: string;
    };
};
export interface GatewayConfiguration {
    mode?: Mode;
    credentials: Credentials;
}
export declare type ClassType<M extends Methods, T extends Types> = M extends "Paypal" ? ReturnType<T extends "One-Time" ? typeof EpicPaypalPaymentsOneTime : () => never> : M extends "Stripe" ? ReturnType<T extends "One-Time" ? typeof EpicStripePaymentsOneTime : () => never> : never;
export declare class Gateway<M extends Methods> {
    protected Method: M;
    protected Configuration?: GatewayConfiguration;
    constructor(Method: M, options?: GatewayConfiguration);
    configure: (options: GatewayConfiguration) => this;
    getMethod: () => M;
    getConfiguration: () => GatewayConfiguration | undefined;
    init: <T extends "One-Time">(type: T) => ClassType<M, T>;
    createInstance: () => Stripe | typeof Paypal;
    createPaypalInstance: () => typeof Paypal;
    createStripeInstance: () => Stripe;
}
