"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Gateway = void 0;
const exceptions_1 = require("./exceptions");
const paypal_rest_sdk_1 = __importDefault(require("paypal-rest-sdk"));
const stripe_1 = __importDefault(require("stripe"));
// Available Gateway Instances
const one_time_1 = require("./paypal/payments/one-time");
const one_time_2 = require("./stripe/payments/one-time");
class Gateway {
    constructor(Method, options) {
        this.Method = Method;
        this.configure = (options) => {
            // Resolve & Validate Mode
            if (!options.mode)
                options.mode = "Test";
            if (!["Live", "Test"].includes(options.mode))
                throw new exceptions_1.GatewayException(`Invalid Mode '${options.mode}' has been provided!`);
            this.Configuration = options;
            return this;
        };
        this.getMethod = () => this.Method;
        this.getConfiguration = () => this.Configuration;
        this.init = (type) => {
            if (typeof this.getConfiguration() !== "object")
                throw new exceptions_1.GatewayException(`No configuration has been provided!`);
            else if (this.getMethod() === "Paypal")
                if (type === "One-Time")
                    return one_time_1.EpicPaypalPaymentsOneTime(this);
                else
                    throw new exceptions_1.GatewayException(`Invalid Instance type '${type}' has been provided!`);
            else if (this.getMethod() === "Stripe")
                if (type === "One-Time")
                    return one_time_2.EpicStripePaymentsOneTime(this);
                else
                    throw new exceptions_1.GatewayException(`Invalid Instance type '${type}' has been provided!`);
            else
                throw new exceptions_1.GatewayException(`Invalid Payment Method '${this.getMethod()}' has been provided!`);
        };
        this.createInstance = () => {
            if (this.getMethod() === "Paypal")
                return this.createPaypalInstance();
            else if (this.getMethod() === "Stripe")
                return this.createStripeInstance();
            else
                throw new exceptions_1.GatewayException(`Invalid Payment Method '${this.getMethod()}' has been provided!`);
        };
        this.createPaypalInstance = () => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            // Get Configuration
            const Configuration = this.getConfiguration();
            // Validate Configuration
            if (!((_b = (_a = Configuration === null || Configuration === void 0 ? void 0 : Configuration.credentials) === null || _a === void 0 ? void 0 : _a[Configuration.mode]) === null || _b === void 0 ? void 0 : _b.PK) ||
                typeof ((_d = (_c = Configuration === null || Configuration === void 0 ? void 0 : Configuration.credentials) === null || _c === void 0 ? void 0 : _c[Configuration.mode]) === null || _d === void 0 ? void 0 : _d.PK) !== "string")
                throw new exceptions_1.GatewayException(`A valid Paypal Client ID is required!`);
            else if (!((_f = (_e = Configuration === null || Configuration === void 0 ? void 0 : Configuration.credentials) === null || _e === void 0 ? void 0 : _e[Configuration.mode]) === null || _f === void 0 ? void 0 : _f.SK) ||
                typeof ((_h = (_g = Configuration === null || Configuration === void 0 ? void 0 : Configuration.credentials) === null || _g === void 0 ? void 0 : _g[Configuration.mode]) === null || _h === void 0 ? void 0 : _h.SK) !== "string")
                throw new exceptions_1.GatewayException(`A valid Paypal Client Secret is required!`);
            // Configure Paypal
            paypal_rest_sdk_1.default.configure({
                mode: Configuration.mode.toLowerCase() === "live" ? "live" : "sandbox",
                client_id: (_j = Configuration.credentials[Configuration.mode]) === null || _j === void 0 ? void 0 : _j.PK,
                client_secret: (_k = Configuration.credentials[Configuration.mode]) === null || _k === void 0 ? void 0 : _k.SK,
            });
            return paypal_rest_sdk_1.default;
        };
        this.createStripeInstance = () => {
            var _a, _b, _c, _d, _e;
            // Get Configuration
            const Configuration = this.getConfiguration();
            // Validate Configuration
            if (!((_b = (_a = Configuration === null || Configuration === void 0 ? void 0 : Configuration.credentials) === null || _a === void 0 ? void 0 : _a[Configuration.mode]) === null || _b === void 0 ? void 0 : _b.SK) ||
                typeof ((_d = (_c = Configuration === null || Configuration === void 0 ? void 0 : Configuration.credentials) === null || _c === void 0 ? void 0 : _c[Configuration.mode]) === null || _d === void 0 ? void 0 : _d.SK) !== "string")
                throw new exceptions_1.GatewayException(`A valid Stripe Secret Key is required!`);
            // Configure Stripe
            return new stripe_1.default((_e = Configuration.credentials[Configuration.mode]) === null || _e === void 0 ? void 0 : _e.SK, {
                apiVersion: "2020-08-27",
            });
        };
        // Add Configuration
        if (typeof options === "object")
            this.configure(options);
    }
}
exports.Gateway = Gateway;
