"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EpicStripePaymentsOneTime = void 0;
const exceptions_1 = require("../../exceptions");
const EpicStripePaymentsOneTime = (Parent) => class {
    constructor(configuration) {
        // Create SDK Instance
        this.Instance = Parent.createInstance();
        this.configure = (configuration) => {
            if (typeof configuration === "object") {
                if (!configuration.returnUrl ||
                    typeof configuration.returnUrl !== "string")
                    // Validate Configuration
                    throw new exceptions_1.GatewayException(`Please provide a valid Return URL!`);
                if (!configuration.cancelUrl ||
                    typeof configuration.cancelUrl !== "string")
                    throw new exceptions_1.GatewayException(`Please provide a valid Cancel URL!`);
                // Save Configuration
                this.Configuration = configuration;
            }
            return this;
        };
        this.create = (Items, custom) => new Promise((resolve, reject) => {
            var _a;
            // Get Items Output
            const Output = Items.get();
            // Create Payment Session
            this.Instance.checkout.sessions
                .create(Object.assign({ mode: "payment", payment_method_types: ((_a = this.Configuration) === null || _a === void 0 ? void 0 : _a.methods) || ["card"], success_url: this.Configuration.returnUrl, cancel_url: this.Configuration.cancelUrl, line_items: Output.items.map((item) => (Object.assign(Object.assign({}, item), { sku: undefined, price: undefined, amount: item.price * 100 }))) }, custom))
                .then((response) => {
                var _a, _b, _c, _d;
                return resolve({
                    method: Parent.getMethod(),
                    mode: ((_a = Parent.getConfiguration()) === null || _a === void 0 ? void 0 : _a.mode) || "Test",
                    key: (_d = (_b = Parent.getConfiguration()) === null || _b === void 0 ? void 0 : _b.credentials[((_c = Parent.getConfiguration()) === null || _c === void 0 ? void 0 : _c.mode) || "Test"]) === null || _d === void 0 ? void 0 : _d.PK,
                    response,
                });
            })
                .catch((error) => reject(new exceptions_1.GatewayException(error)));
        });
        this.execute = (options) => new Promise(async (resolve, reject) => {
            var _a, _b;
            try {
                // Fetch Session
                const Session = await this.Instance.checkout.sessions.retrieve(options.paymentId);
                // Check Payment Status
                if (Session.payment_status === "paid") {
                    resolve({
                        method: Parent.getMethod(),
                        mode: ((_a = Parent.getConfiguration()) === null || _a === void 0 ? void 0 : _a.mode) || "Test",
                        details: {
                            currency: (_b = Session.currency) === null || _b === void 0 ? void 0 : _b.toLowerCase(),
                            amount: (Session.amount_total || 0) / 100,
                        },
                        response: Session,
                    });
                }
                else
                    throw new Error(`Payment has been failed!`);
            }
            catch (error) {
                reject(new exceptions_1.GatewayException(error));
            }
        });
        this.configure(configuration);
    }
};
exports.EpicStripePaymentsOneTime = EpicStripePaymentsOneTime;
